@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2010-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,12 +7,16 @@
 #ifndef ISC_DATA_H
 #define ISC_DATA_H 1
 
-#include <stdint.h>
+#include <iostream>
+#include <map>
+#include <stdexcept>
 #include <string>
 #include <vector>
-#include <map>
+
 #include <boost/shared_ptr.hpp>
-#include <stdexcept>
+
+#include <stdint.h>
+
 #include <exceptions/exceptions.h>
 
 namespace isc { namespace data {
@@ -263,8 +267,6 @@ public:
     virtual bool setValue(const std::map<std::string, ConstElementPtr>& v);
     //@}
 
-
-
     // Other functions for specific subtypes
 
     /// @name ListElement functions
@@ -326,7 +328,7 @@ public:
     virtual void remove(const std::string& name);
 
     /// Checks if there is data at the given key
-    /// @param name The key of the Element to remove
+    /// @param name The key of the Element checked for existence
     /// @return true if there is data at the key, false if not.
     virtual bool contains(const std::string& name) const;
 
@@ -352,7 +354,6 @@ public:
     virtual bool find(const std::string& identifier, ConstElementPtr& t) const;
     //@}
 
-
     /// @name Factory functions
 
     // TODO: should we move all factory functions to a different class
@@ -379,6 +380,8 @@ public:
                              const Position& pos = ZERO_POSITION());
     static ElementPtr create(const long int i,
                              const Position& pos = ZERO_POSITION());
+    static ElementPtr create(const uint32_t i,
+                             const Position& pos = ZERO_POSITION());
     static ElementPtr create(const double d,
                              const Position& pos = ZERO_POSITION());
     static ElementPtr create(const bool b,
@@ -403,7 +406,6 @@ public:
     static ElementPtr createMap(const Position& pos = ZERO_POSITION());
     //@}
 
-
     /// @name Compound factory functions
 
     /// @brief These functions will parse the given string (JSON)
@@ -425,10 +427,10 @@ public:
     /// @param in The string to parse the element from
     /// @param preproc specified whether preprocessing (e.g. comment removal)
     ///                should be performed
+    /// @throw JSONError
     /// @return An ElementPtr that contains the element(s) specified
     /// in the given input stream.
-    static ElementPtr fromJSON(std::istream& in, bool preproc = false)
-        throw(JSONError);
+    static ElementPtr fromJSON(std::istream& in, bool preproc = false);
 
     /// Creates an Element from the given input stream containing JSON
     /// formatted data.
@@ -437,11 +439,12 @@ public:
     /// @param file_name specified input file name (used in error reporting)
     /// @param preproc specified whether preprocessing (e.g. comment removal)
     ///                should be performed
+    /// @throw JSONError
     /// @return An ElementPtr that contains the element(s) specified
     /// in the given input stream.
+    /// @throw JSONError
     static ElementPtr fromJSON(std::istream& in, const std::string& file_name,
-                               bool preproc = false)
-        throw(JSONError);
+                               bool preproc = false);
 
     /// Creates an Element from the given input stream, where we keep
     /// track of the location in the stream for error reporting.
@@ -452,12 +455,13 @@ public:
     /// track of the current line.
     /// @param pos A reference to the int where the function keeps
     /// track of the current position within the current line.
+    /// @throw JSONError
     /// @return An ElementPtr that contains the element(s) specified
     /// in the given input stream.
     // make this one private?
+    /// @throw JSONError
     static ElementPtr fromJSON(std::istream& in, const std::string& file,
-                               int& line, int &pos)
-        throw(JSONError);
+                               int& line, int &pos);
 
     /// Reads contents of specified file and interprets it as JSON.
     ///
@@ -527,6 +531,50 @@ public:
     /// @return ElementPtr with the data that is parsed.
     static ElementPtr fromWire(const std::string& s);
     //@}
+
+    /// @brief Remove all empty maps and lists from this Element and its
+    /// descendants.
+    void removeEmptyContainersRecursively() {
+        if (type_ == list || type_ == map) {
+            size_t s(size());
+            for (size_t i = 0; i < s; ++i) {
+                // Get child.
+                ElementPtr child;
+                if (type_ == list) {
+                    child = getNonConst(i);
+                } else if (type_ == map) {
+                    std::string const key(get(i)->stringValue());
+                    // The ElementPtr - ConstElementPtr disparity between
+                    // ListElement and MapElement is forcing a const cast here.
+                    // It's undefined behavior to modify it after const casting.
+                    // The options are limited. I've tried templating, moving
+                    // this function from a member function to free-standing and
+                    // taking the Element template as argument. I've tried
+                    // making it a virtual function with overridden
+                    // implementations in ListElement and MapElement. Nothing
+                    // works.
+                    child = boost::const_pointer_cast<Element>(get(key));
+                }
+
+                // Makes no sense to continue for non-container children.
+                if (child->getType() != list && child->getType() != map) {
+                    continue;
+                }
+
+                // Recurse if not empty.
+                if (!child->empty()){
+                    child->removeEmptyContainersRecursively();
+                }
+
+                // When returning from recursion, remove if empty.
+                if (child->empty()) {
+                    remove(i);
+                    --i;
+                    --s;
+                }
+            }
+        }
+    }
 };
 
 /// Notes: IntElement type is changed to int64_t.
@@ -541,8 +589,6 @@ public:
 ///
 class IntElement : public Element {
     int64_t i;
-private:
-
 public:
     IntElement(int64_t v, const Position& pos = ZERO_POSITION())
         : Element(integer, pos), i(v) { }
@@ -639,6 +685,17 @@ public:
     size_t size() const { return (l.size()); }
     bool empty() const { return (l.empty()); }
     bool equals(const Element& other) const;
+
+    /// @brief Sorts the elements inside the list.
+    ///
+    /// The list must contain elements of the same type.
+    /// Call with the key by which you want to sort when the list contains maps.
+    /// Nested lists are not supported.
+    /// Call without a parameter when sorting any other type.
+    ///
+    /// @param index the key by which you want to sort when the list contains
+    /// maps
+    void sort(std::string const& index = std::string());
 };
 
 class MapElement : public Element {
@@ -648,53 +705,79 @@ public:
     MapElement(const Position& pos = ZERO_POSITION()) : Element(map, pos) {}
     // @todo should we have direct iterators instead of exposing the std::map
     // here?
-    const std::map<std::string, ConstElementPtr>& mapValue() const {
+    const std::map<std::string, ConstElementPtr>& mapValue() const override {
         return (m);
     }
     using Element::getValue;
-    bool getValue(std::map<std::string, ConstElementPtr>& t) const {
+    bool getValue(std::map<std::string, ConstElementPtr>& t) const override {
         t = m;
         return (true);
     }
     using Element::setValue;
-    bool setValue(const std::map<std::string, ConstElementPtr>& v) {
+    bool setValue(const std::map<std::string, ConstElementPtr>& v) override {
         m = v;
         return (true);
     }
     using Element::get;
-    ConstElementPtr get(const std::string& s) const {
-        return (contains(s) ? m.find(s)->second : ConstElementPtr());
+    ConstElementPtr get(const std::string& s) const override {
+        auto found = m.find(s);
+        return (found != m.end() ? found->second : ConstElementPtr());
     }
+
+    /// @brief Get the i-th element in the map.
+    ///
+    /// Useful when required to iterate with an index.
+    ///
+    /// @param i the position of the element you want to return
+    /// @return the element at position i
+    ConstElementPtr get(int const i) const override {
+        auto it(m.begin());
+        std::advance(it, i);
+        return create(it->first);
+    }
+
     using Element::set;
-    void set(const std::string& key, ConstElementPtr value);
+    void set(const std::string& key, ConstElementPtr value) override;
     using Element::remove;
-    void remove(const std::string& s) { m.erase(s); }
-    bool contains(const std::string& s) const {
+    void remove(const std::string& s) override { m.erase(s); }
+
+    /// @brief Remove the i-th element from the map.
+    ///
+    /// @param i the position of the element you want to remove
+    void remove(int const i) override {
+        auto it(m.begin());
+        std::advance(it, i);
+        m.erase(it);
+    }
+
+    bool contains(const std::string& s) const override {
         return (m.find(s) != m.end());
     }
-    void toJSON(std::ostream& ss) const;
+    void toJSON(std::ostream& ss) const override;
 
     // we should name the two finds better...
     // find the element at id; raises TypeError if one of the
     // elements at path except the one we're looking for is not a
     // mapelement.
     // returns an empty element if the item could not be found
-    ConstElementPtr find(const std::string& id) const;
+    ConstElementPtr find(const std::string& id) const override;
 
     // find the Element at 'id', and store the element pointer in t
     // returns true if found, or false if not found (either because
     // it doesn't exist or one of the elements in the path is not
     // a MapElement)
-    bool find(const std::string& id, ConstElementPtr& t) const;
+    bool find(const std::string& id, ConstElementPtr& t) const override;
 
     /// @brief Returns number of stored elements
     ///
     /// @return number of elements.
-    size_t size() const {
+    size_t size() const override {
         return (m.size());
     }
 
-    bool equals(const Element& other) const;
+    bool equals(const Element& other) const override;
+
+    bool empty() const override { return (m.empty()); }
 };
 
 /// Checks whether the given ElementPtr is a NULL pointer
@@ -718,20 +801,125 @@ void removeIdentical(ElementPtr a, ConstElementPtr b);
 /// Raises a TypeError if a or b are not MapElements
 ConstElementPtr removeIdentical(ConstElementPtr a, ConstElementPtr b);
 
-/// @brief Merges the data from other into element.
-/// (on the first level). Both elements must be
-/// MapElements.
-/// Every string,value pair in other is copied into element
-/// (the ElementPtr of value is copied, this is not a new object)
-/// Unless the value is a NullElement, in which case the
-/// key is removed from element, rather than setting the value to
-/// the given NullElement.
-/// This way, we can remove values from for instance maps with
-/// configuration data (which would then result in reverting back
-/// to the default).
+/// @brief Merges the data from other into element. (on the first level). Both
+/// elements must be MapElements. Every string, value pair in other is copied
+/// into element (the ElementPtr of value is copied, this is not a new object)
+/// Unless the value is a NullElement, in which case the key is removed from
+/// element, rather than setting the value to the given NullElement.
+/// This way, we can remove values from for instance maps with configuration
+/// data (which would then result in reverting back to the default).
 /// Raises a TypeError if either ElementPtr is not a MapElement
 void merge(ElementPtr element, ConstElementPtr other);
 
+/// @brief Function used to check if two MapElements refer to the same
+/// configuration data. It can check if the two MapElements have the same or
+/// have equivalent value for some members.
+/// e.g.
+/// (
+///  left->get("prefix")->stringValue() == right->get("prefix")->stringValue() &&
+///  left->get("prefix-len")->intValue() == right->get("prefix-len")->intValue() &&
+///  left->get("delegated-len")->intValue() == right->get("delegated-len")->intValue()
+/// )
+typedef std::function<bool (ElementPtr&, ElementPtr&)> MatchTestFunc;
+
+/// @brief Function used to check if the data provided for the element contains
+/// only information used for identification, or it contains extra useful data.
+typedef std::function<bool (ElementPtr&)> NoDataTestFunc;
+
+/// @brief Function used to check if the key is used for identification
+typedef std::function<bool (const std::string&)> IsKeyTestFunc;
+
+/// @brief Structure holding the test functions used to traverse the element
+/// hierarchy.
+struct HierarchyTraversalTest {
+    MatchTestFunc match_;
+    NoDataTestFunc no_data_;
+    IsKeyTestFunc is_key_;
+};
+
+/// @brief Mapping between a container name and functions used to match elements
+/// inside the container.
+typedef std::map<std::string, HierarchyTraversalTest> FunctionMap;
+
+/// @brief Hierarchy descriptor of the containers in a specific Element
+/// hierarchy tree. The position inside the vector indicates the level at which
+/// the respective containers are located.
+///
+/// e.g.
+/// {
+///   { { "pools", { ... , ... } }, { "pd-pools", { ... , ... } }, { "option-data", { ... , ... } } },
+///   { { "option-data", { ... , ... } } }
+/// }
+/// At first subnet level the 'pools', 'pd-pools' and 'option-data' containers
+/// can be found.
+/// At second subnet level the 'option-data' container can be found
+/// (obviously only inside 'pools' and 'pd-pools' containers).
+typedef std::vector<FunctionMap> HierarchyDescriptor;
+
+/// @brief Merges the diff data by adding the missing elements from 'other'
+/// to 'element' (recursively). Both elements must be the same Element type.
+/// Raises a TypeError if elements are not the same Element type.
+/// @note
+/// for non map and list elements the values are updated with the new values
+/// for maps:
+///     - non map and list elements are added/updated with the new values
+///     - list and map elements are processed recursively
+/// for lists:
+///     - regardless of the element type, all elements from 'other' are added to
+///       'element'
+///
+/// @param element The element to which new data is added.
+/// @param other The element containing the data which needs to be added.
+/// @param hierarchy The hierarchy describing the elements relations and
+/// identification keys.
+/// @param key The container holding the current element.
+/// @param idx The level inside the hierarchy the current element is located.
+void mergeDiffAdd(ElementPtr& element, ElementPtr& other,
+                  HierarchyDescriptor& hierarchy, std::string key,
+                  size_t idx = 0);
+
+/// @brief Merges the diff data by removing the data present in 'other' from
+/// 'element' (recursively). Both elements must be the same Element type.
+/// Raises a TypeError if elements are not the same Element type.
+/// for non map and list elements the values are set to NullElement
+/// for maps:
+///     - non map and list elements are removed from the map
+///     - list and map elements are processed recursively
+/// for lists:
+///     - regardless of the element type, all elements from 'other' matching
+///       elements in 'element' are removed
+///
+/// @param element The element from which new data is removed.
+/// @param other The element containing the data which needs to be removed.
+/// @param hierarchy The hierarchy describing the elements relations and
+/// identification keys.
+/// @param key The container holding the current element.
+/// @param idx The level inside the hierarchy the current element is located.
+void mergeDiffDel(ElementPtr& element, ElementPtr& other,
+                  HierarchyDescriptor& hierarchy, std::string key,
+                  size_t idx = 0);
+
+/// @brief Extends data by adding the specified 'extension' elements from
+/// 'other' inside the 'container' element (recursively). Both elements must be
+/// the same Element type.
+/// Raises a TypeError if elements are not the same Element type.
+///
+/// @param container The container holding the data that must be extended.
+/// @param extension The name of the element that contains the data that must be
+/// added (if not already present) in order to extend the initial data.
+/// @param element The element from which new data is added.
+/// @param other The element containing the data which needs to be added.
+/// @param hierarchy The hierarchy describing the elements relations and
+/// identification keys.
+/// @param key The container holding the current element.
+/// @param idx The level inside the hierarchy the current element is located.
+/// @param alter The flag which indicates if the current element should be
+/// updated.
+void extend(const std::string& container, const std::string& extension,
+            ElementPtr& element, ElementPtr& other,
+            HierarchyDescriptor& hierarchy, std::string key, size_t idx = 0,
+            bool alter = false);
+
 /// @brief Copy the data up to a nesting level.
 ///
 /// The copy is a deep copy so nothing is shared if it is not
@@ -741,8 +929,8 @@ void merge(ElementPtr element, ConstElementPtr other);
 /// @param level nesting level (default is 100, 0 means shallow copy,
 /// negative means outbound and perhaps looping forever).
 /// @return a pointer to a fresh copy
-/// \throw raises a BadValue is a null pointer occurs.
-ElementPtr copy(ConstElementPtr from, int level = 100); 
+/// @throw raises a BadValue is a null pointer occurs.
+ElementPtr copy(ConstElementPtr from, int level = 100);
 
 /// @brief Compares the data with other using unordered lists
 ///
@@ -756,6 +944,7 @@ bool isEquivalent(ConstElementPtr a, ConstElementPtr b);
 /// This operator converts the @c ConstElementPtr into a string and
 /// inserts it into the output stream @c out with an initial
 /// indentation @c indent and add at each level @c step spaces.
+/// For maps if there is a comment property it is printed first.
 ///
 /// @param element A @c ConstElementPtr to pretty print
 /// @param out A @c std::ostream on which the print operation is performed
@@ -768,6 +957,7 @@ void prettyPrint(ConstElementPtr element, std::ostream& out,
 ///
 /// This operator converts the @c ConstElementPtr into a string with
 /// an initial indentation @c indent and add at each level @c step spaces.
+/// For maps if there is a comment property it is printed first.
 ///
 /// @param element A @c ConstElementPtr to pretty print
 /// @param indent An initial number of spaces to add each new line
@@ -776,7 +966,6 @@ void prettyPrint(ConstElementPtr element, std::ostream& out,
 std::string prettyPrint(ConstElementPtr element,
                         unsigned indent = 0, unsigned step = 2);
 
-///
 /// @brief Insert Element::Position as a string into stream.
 ///
 /// This operator converts the @c Element::Position into a string and
@@ -789,7 +978,6 @@ std::string prettyPrint(ConstElementPtr element,
 /// parameter @c out after the insertion operation.
 std::ostream& operator<<(std::ostream& out, const Element::Position& pos);
 
-///
 /// @brief Insert the Element as a string into stream.
 ///
 /// This method converts the @c ElementPtr into a string with
@@ -808,7 +996,11 @@ std::ostream& operator<<(std::ostream& out, const Element& e);
 
 bool operator==(const Element& a, const Element& b);
 bool operator!=(const Element& a, const Element& b);
-} }
+bool operator<(const Element& a, const Element& b);
+
+}  // namespace data
+}  // namespace isc
+
 #endif // ISC_DATA_H
 
 // Local Variables:
